@@ -96,7 +96,7 @@ sub wm_scandir {
 sub wm_installtheme {
   my ($tempdir, $filename, $defname) = @_;
   my ($dest_style, $dest_bg, $rootcommand_destbg,
-    @files, @filetypes, $i, $stylefile, $bgimage, $theme,
+    $menutext, @files, @filetypes, $i, $stylefile, $bgimage, $theme,
     $includedest, $specified_bg, $specified_base, @bgcandidates);
 
   getversion_blackbox();
@@ -142,7 +142,10 @@ sub wm_installtheme {
     $dest_style = $globalthemedir;
   }
 
-  dbugout("style dir is $dest_style", "bg dir is $dest_bg") if $debug;
+  if ($debug) {
+    dbugout("WM_INSTALLTHEME: style dir is $dest_style");
+    dbugout("WM_INSTALLTHEME: bg dir is $dest_bg");
+  }
 
    #=========================================================================
    # Examine files and move to destinations
@@ -174,17 +177,22 @@ sub wm_installtheme {
     }
   }
 
-  choke('theme contains no style file') unless $stylefile;
-
-  dbugout("style file is $stylefile") if $debug;
+  if (!$stylefile) {
+    choke('theme contains no style file');
+  } elsif ($debug) {
+    dbugout("WM_INSTALLTHEME: style file is $stylefile");
+  }
 
   if ($defname) {
     $theme = $defname;
   } else {
     ($theme = $stylefile) =~ s!.*/!!;
-    ($theme = $filename) =~ s/\..*$// unless $theme;
-    choke("strange, I cannot get a theme name from \"$stylefile\" or \"$filename\"")
-      unless $theme;
+    if (!$theme) {
+      ($theme = $filename) =~ s!\..*$!!;
+    }
+    if (!$theme) {
+      choke("strange, I cannot get a theme name from \"$stylefile\" or \"$filename\"");
+    }
   }
   $theme = checkname($theme);
 
@@ -199,22 +207,23 @@ sub wm_installtheme {
 
   if (!$>) {
     if (-f "$dest_style/stylesmenu") {
-      stowfile("$dest_style/stylesmenu",
-        styleentry_blackbox($theme) . slurpfile("$dest_style/stylesmenu"));
+      $menutext = styleentry_blackbox($theme) .
+        slurpfile("$dest_style/stylesmenu");
+      stowfile("$dest_style/stylesmenu", $menutext);
     }
   } elsif (!$dirusage{$dest_style}) {
 
-    dbugout("style goes into a _new_ [stylesdir]") if $debug;
+    dbugout("WM_INSTALLTHEME: style goes into a _new_ [stylesdir]") if $debug;
     addtostylesmenu_blackbox("    [stylesdir] (" .
       bbmenuescape($dest_style) . ")\n");
 
   } elsif ($dirusage{$dest_style} =~ /^include:(.+)/) {
 
     $includedest = $1;
-    dbugout("menu item goes into file $includedest") if $debug;
-
-    stowfile($includedest,
-      styleentry_blackbox($theme) . slurpfile($includedest));
+    dbugout("WM_INSTALLTHEME: menu item goes into file $includedest") if $debug;
+    $menutext = styleentry_blackbox($theme) .
+      slurpfile($includedest);
+    stowfile($includedest, $menutext);
 
       ## If this included fragment was empty, Blackbox won't check it
       ## and the new theme won't show up in the menu.  By touching
@@ -223,8 +232,7 @@ sub wm_installtheme {
 
       ## Since [include]s that actually contain something will be
       ## checked, we won't need to touch anything when uninstalling.
-    my $now = time;
-    utime $now, $now, $mainmenu;
+    touch($mainmenu);  
 
   } elsif ($dirusage{$dest_style} eq 'style') {
 
@@ -232,7 +240,8 @@ sub wm_installtheme {
 
   } elsif ($dirusage{$dest_style} eq 'stylesdir') {
 
-    dbugout("style goes into an existing [stylesdir]") if $debug;
+    dbugout("WM_INSTALLTHEME: style goes into an existing [stylesdir]")
+      if $debug;
 
   } else {
 
@@ -251,8 +260,11 @@ sub wm_activatetheme {
   # abort if $blackboxrc not writable
 
   unless (-w $blackboxrc) {
-    choke("Can't activate themes: $blackboxrc isn't writable.") if $explicit;
-    return;
+    if ($explicit) {
+      choke("Can't activate themes: $blackboxrc isn't writable.");
+    } else {
+      return;
+    }
   }
 
   # modify it to indicate desired theme
@@ -266,7 +278,7 @@ sub wm_activatetheme {
 
   # send signal
 
-  sigbyname(USR1, $wm_executable);
+  sigbyname('USR1', $wm_executable);
 }
 
   ##
@@ -292,7 +304,7 @@ sub wm_uninstalltheme {
 
   if ($bgimage = identifybgfile_blackbox($themes{$theme}{path})) {
     unlink expandhomedir($bgimage);
-    dbugout("erased background image $bgimage") if $debug;
+    dbugout("WM_UNINSTALLTHEME: erased background image $bgimage") if $debug;
   }
 
   unlink $themes{$theme}{path};
@@ -350,7 +362,8 @@ sub wm_versionok {
 
   if ($blackbox_version and $request =~ /(\d+\.\d+)/) {
     $rqver = $1;
-    dbugout("checking $rqver against Blackbox v$blackbox_version") if $debug;
+    dbugout("WM_VERSIONOK: checking $rqver against Blackbox v$blackbox_version")
+      if $debug;
     if ($blackbox_version < 0.6) {
       return $rqver < 0.6;
     } elsif ($blackbox_version < 0.7) {
@@ -360,7 +373,8 @@ sub wm_versionok {
       return 1;
     }
   } else {
-    dbugout("incomprehensible version \"$request\": returning ok") if $debug;
+    dbugout("WM_VERSIONOK: incomprehensible version \"$request\": returning ok")
+      if $debug;
     return 1;
   }
 }
@@ -377,7 +391,7 @@ sub fixstyle_blackbox {
     $bgcmd, $err, $foundrootcmd, $true_bgimage, $valid_rootcommands);
 
   open F, $stylefile or return;
-  while (defined ($line = readcontinuationline(\*F, 0))) {
+  while (defined ($line = nextstyleline_blackbox(\*F, 0))) {
 
     if ($foundrootcmd) {
 
@@ -403,7 +417,8 @@ sub fixstyle_blackbox {
             ($bgbase = $_) =~ s!.*/!!;
             if ($bgbase and $bgbase eq $imgbase) {
               $true_bgimage = $_;
-              dbugout("background image is $true_bgimage") if $debug;
+              dbugout("FIXSTYLE_BLACKBOX: background image is $true_bgimage")
+                if $debug;
               last;
             }
           }
@@ -473,18 +488,18 @@ sub fixstyle_blackbox {
 sub parserootcommand_blackbox {
   my $rootcommand = shift;
 
-  dbugout("looking at $rootcommand") if $debug;
+  dbugout("PRC: looking at $rootcommand") if $debug;
 
   $rootcommand =~ s!^/\S*/!!;  # remove path from utility
 
   if ($rootcommand =~ /(^|^.+?\s)((?:~|\.?\.?\/).+?\/.+|\/.+)(\s.+|)$/) {
-    dbugout("returning (1)  [$1] [$3] [$2]") if $debug;
+    dbugout("PRC: returning (1)  [$1] [$3] [$2]") if $debug;
     return $1, $3, $2;
   } elsif ($rootcommand =~ /(^|^.+?\s")((?:~|\.?\.?\/).+?\/.+|\/.+[^\\])("\s.+|")$/) {
-    dbugout("returning (2)  [$1] [$3] [$2]") if $debug;
+    dbugout("PRC: returning (2)  [$1] [$3] [$2]") if $debug;
     return $1, $3, $2;
   }
-  dbugout("didn't match.") if $debug;
+  dbugout("PRC: didn't match.") if $debug;
   ('', '', '');
 }
 
@@ -625,7 +640,7 @@ sub scanmenu_blackbox {
   my ($menutext, @includes, $line, $dir);
 
   open (F, $menu) or return;
-  dbugout("reading $menu") if $debug;
+  dbugout("SCANMENU_BLACKBOX: reading $menu") if $debug;
 
   while (defined($line = <F>)) {
     next if $line =~ /^\s*#/;
@@ -638,7 +653,7 @@ sub scanmenu_blackbox {
       ($dir = $path) =~ s!/[^/]*$!!;
 
       if (-f $path) {
-        dbugout("adding \"$name\" from [style] entry") if $debug;
+        dbugout("SCANMENU_BLACKBOX: adding \"$name\" from [style] entry") if $debug;
         setdirusage_blackbox($dir, 'style', '') if $menu eq $mainmenu;
         createthemerec($name, $path, (stat _)[4]);
         $menufile{$name} = $menu;
@@ -646,7 +661,7 @@ sub scanmenu_blackbox {
         setdirusage_blackbox($dir, 'style', '');
         $$styledirs{$dir} = 1;
       } else {
-        dbugout("not adding \"$name\" from invalid [style] entry")
+        dbugout("SCANMENU_BLACKBOX: not adding \"$name\" from invalid [style] entry")
           if $debug;
       }
     } elsif ($line =~ /\[\s*include\s*\]\s*\((.+)\)\s*$/) {
@@ -657,7 +672,7 @@ sub scanmenu_blackbox {
       if ($$rccheck{$path}) {
         complain("warning: repeated [include] for \"$path\"");
       } elsif (-f $path) {
-        dbugout("include $path") if $debug;
+        dbugout("SCANMENU_BLACKBOX: include $path") if $debug;
         setdirusage_blackbox($dir, 'include', $path) if $menu eq $mainmenu;
         push @includes, $path;
         $$rccheck{$path} = 1;
@@ -675,11 +690,11 @@ sub scanmenu_blackbox {
         print "$programname: restart Blackbox so it will scan this directory.\n";
       }
       if (-d $path) {
-        dbugout("stylesdir $path") if $debug;
+        dbugout("SCANMENU_BLACKBOX: stylesdir $path") if $debug;
         setdirusage_blackbox($path, 'stylesdir', '');
         $$styledirs{$path} = 1;
       } else {
-        dbugout("not including invalid stylesdir $path")
+        dbugout("SCANMENU_BLACKBOX: not including invalid stylesdir $path")
           if $debug;
       }
     }
@@ -714,12 +729,14 @@ sub setdirusage_blackbox {
 sub getversion_blackbox {
   return if defined $blackbox_version;
 
-  if (`blackbox -version` =~ /\b((\d+)\.(\d+))/) {
+  if (`blackbox -version` =~ /^\S+\s+((\d+)\.(\d+))/) {
     $blackbox_version = $1;
-    dbugout("version recorded as $blackbox_version") if $debug;
+    dbugout("GETVERSION_BLACKBOX: version recorded as $blackbox_version")
+      if $debug;
   } else {
     $blackbox_version = 0;
-    dbugout("couldn't determine version!") if $debug;
+    dbugout("GETVERSION_BLACKBOX: couldn't determine version!")
+      if $debug;
   }
 }
 
@@ -734,7 +751,7 @@ sub identifybgfile_blackbox {
   my $line;
 
   open F, $stylefile or return;
-  while (defined ($line = readcontinuationline(\*F, 1))) {
+  while (defined ($line = nextstyleline_blackbox(\*F, 1))) {
     if ($line =~ /^\s*rootcommand\s*:\s*(.+?)\s*$/i) {
       (undef, undef, $bgimage) = parserootcommand_blackbox($1);
       last;
@@ -744,6 +761,20 @@ sub identifybgfile_blackbox {
   $bgimage;
 }
 
+
+sub nextstyleline_blackbox {
+  my ($fh, $stripbs) = @_;
+  my ($line, $in);
+
+  return unless defined($line = <$fh>);
+
+  $line .= $in while $line =~ /\\$/ and defined($in = <$fh>);
+
+  $line =~ s/\\(?:\n|$)//gs if $stripbs;
+
+  $line;
+}
+
 sub bbmenuescape {
   my $text = shift;
   $text =~ s/([(){}])/\\$1/g;
