@@ -26,11 +26,10 @@ checkuserinstall_icewm();
 sub wm_getdefaultdirs {
   verify_themedirs(0,
     "$home/.icewm/themes",
-    '/usr/local/share/icewm/themes',     #  DEFAULT
-    '/usr/share/icewm/themes',           #  alternate default
-    '/usr/local/lib/X11/icewm/themes',   #  old default
+    '/usr/local/lib/X11/icewm/themes',   #  DEFAULT
     '/usr/lib/X11/icewm/themes',         #  Debian
     '/usr/X11R6/lib/X11/icewm/themes',   #  Red Hat
+    '/usr/local/share/icewm/themes',     #  A guess
     '/usr/X11R6/lib/icewm/themes',       #  What Slackware would probably do
     '/usr/X11R6/share/icewm/themes'      #  Or this
     );
@@ -46,7 +45,7 @@ sub wm_scandir {
   my $dir = shift;
   my (%scandirs, $rdir, $mtime);
 
-  dbugout("scanning directory $dir") if $debug;
+  dbugout("WM_SCANDIR: scanning directory $dir") if $debug;
   opendir DH, $dir or choke("can't opendir $dir: $!");
 
     # Directories coming from the cache may not be the global/local
@@ -84,7 +83,7 @@ sub wm_scandir {
 }
 
 sub registerthemedir_icewm {
-  my ($dir, $isnew) = @_;
+  my $dir = shift;
   my $oid = (stat $dir)[4];
   my $mtime = (stat _)[9];
   my ($entry, $name);
@@ -99,17 +98,16 @@ sub registerthemedir_icewm {
    #    plopped into the themes dir)
    #
 
-  dbugout("scanning directory $dir") if $debug;
+  dbugout("REGDIR_ICEWM: scanning directory $dir") if $debug;
   opendir STD, $dir or choke("can't opendir $dir: $!");
   while (defined($entry = readdir STD)) {
     if (!-d "$dir/$entry" and $entry =~ /(.+)\.theme$/) {
       if ($1 eq 'default') {
-        dbugout("keeping default of \"$name\"") if $debug;
+        dbugout("REGDIR_ICEWM: keeping Default of \"$name\"") if $debug;
         createthemerec($name, $dir, $oid);
       } else {
-        dbugout("keeping \"$name/$1\"") if $debug;
+        dbugout("REGDIR_ICEWM: keeping \"$name/$1\"") if $debug;
         createthemerec("$name/$1", "$dir/$entry", $oid);
-        cache_themedir("$name/$1"), $isnew = 0 if $isnew;
       }
     }
   }
@@ -157,7 +155,7 @@ sub wm_installtheme {
   $themename = checkname($possiblename);
 
   syscmd('mv', $cdir, "$destthemedir/$themename");
-  registerthemedir_icewm("$destthemedir/$themename", 1);
+  registerthemedir_icewm("$destthemedir/$themename");
 
   $themename;
 }
@@ -193,15 +191,13 @@ sub wm_activatetheme {
     choke("internal error: can't grok theme file $path");
   }
 
-  if ($themename =~ m!/!) {
-    $tag = qq[Theme="$themename.theme"\n];
+  $prefs = slurpfile("$home/.icewm/preferences");
+  if ($prefs =~ s/^\s*Theme\s*=.*$/Theme=\"$themefile\"/m) {
   } else {
-    $tag = qq[Theme="$themename/default.theme"\n];
+    $prefs .= "\nTheme=\"$themefile\"\n";
   }
-
-  stowfile("$home/.icewm/theme", $tag);
-
-  sigbyname(HUP, $wm_executable);
+  stowfile("$home/.icewm/preferences", $prefs);
+  sigbyname('HUP', $wm_executable);
 }
 
   ###
@@ -310,7 +306,7 @@ sub wm_versionok {
   my $rqver = shift;
   my $iver;
 
-  dbugout("judging version \"$rqver\"") if $debug;
+  dbugout("WM_VERSIONOK: judging version \"$rqver\"") if $debug;
   $rqver =~ /(\d+(\.\d+))/ and $rqver = $1;
   if ($iver = getversion_icewm()) {
     if ($iver < 0.9) {
@@ -330,10 +326,10 @@ sub getversion_icewm {
     my $icever = `icewm -v 2>&1`;
     if ($icever =~ /icewm\s+(\d+\.\d+)/i) {
       $icewm_version = $1;
-      dbugout("dealing with IceWM version $icewm_version")
+      dbugout("GETVERSION_ICEWM: dealing with IceWM version $icewm_version")
         if $debug;
     } else {
-      dbugout("got incomprehensible \"$icever\" from icewm -v")
+      dbugout("GETVERSION_ICEWM: got incomprehensible \"$icever\" from icewm -v")
         if $debug;
     }
   }
@@ -344,6 +340,7 @@ sub checkuserinstall_icewm {
   return if $icewm_userinstalled;
 
   if (-d "$home/.icewm" and
+      -f "$home/.icewm/preferences" and
       -d "$home/.icewm/themes") {
     $icewm_userinstalled = 1;
   }
@@ -354,7 +351,8 @@ sub userinstall_icewm {
 
   my ($answer, $preffile);
 
-  print "Local IceWM directories are required.  Shall I create them? [Y/n] ";
+  print "Local IceWM configuration files in ~/.icewm are required. Shall I create this\n";
+  print "directory and copy files into it? [Y/n] ";
   $answer = <STDIN>;
 
   if ($answer =~ /^([yY]|\s*$)/) {
@@ -371,6 +369,24 @@ sub userinstall_icewm {
         choke("can't mkdir $home/.icewm/themes: $!");
     }
 
+    if (! -f "$home/.icewm/preferences") {
+
+      if ($globalthemedir and
+          $globalthemedir =~ m|(.+)/[^/]+$|
+          and -f "$1/preferences") {
+        $preffile = "$1/preferences";
+
+      } elsif (-f '/etc/X11/icewm/preferences') {
+         ##  - Debian puts preferences here, separate from the global themes dir
+        $preffile = '/etc/X11/icewm/preferences';
+
+      } else {
+        choke("can't find the system-wide preferences file!");
+      }
+
+      copyfile($preffile, "$home/.icewm/preferences");
+    }
+
     $localthemedir = "$home/.icewm/themes";
     $icewm_userinstalled = 1;
 
