@@ -1,8 +1,9 @@
 #  Download subs for wmtheme
 
 use Socket;
-my $ftp_command;
+my ($ftp_command, %proxy_auth);
 my $hostmatch = '(?i)[a-z](?:[a-z\d-]{0,61}[a-z\d])?(?:\.[a-z](?:[a-z\d-]{0,61}[a-z\d])?)*';
+my $pa_str = "Proxy-Authorization: Basic ";
 
 sub dlquery {
   my ($themetxt, $tosite) = @_;
@@ -107,8 +108,8 @@ sub rdfparse {
   my ($xtext, %searchterms) = @_;
   my ($itemtext, $themename, %qthemes, $keepver, $verblock, $vers, $term);
 
-  ITEM: while ($xtext =~ /<item(\s+[^>]*)?>(.+?)<\/item>/gs) {
-    $itemtext = $2;
+  ITEM: while ($xtext =~ /<item(?:\s+[^>]*)?>(.+?)<\/item>/gs) {
+    $itemtext = $1;
     foreach $term (keys %searchterms) {
       if (entity_text($itemtext, $term) !~ /\Q$searchterms{$term}\E/i) {
         next ITEM;
@@ -261,16 +262,13 @@ sub downloadfile {
 
       ## -  SIZE  - sometimes available in http  - ##
 
-    if ($header =~ /^content-length:\s+(\d+)/im) {
-      $size = $1;
-    } else {
-      $size = 0;
-    }
+    $size = $header->{'content-length'} || 0;
 
       ## -  FILENAME  - from url or header in http  - ##
 
     if ($destination) {
-      if ($header =~ /^content-disposition:.*filename=\s*(.+?)\s/is) {
+      if (exists $header->{'content-disposition'} and
+          $header->{'content-disposition'} =~ /filename=\s*(.+)/i) {
         $filename = $1;
       } elsif ($$loc{location} =~ m!/([^/]+)$!) {
         $filename = cgidecode($1);
@@ -408,40 +406,39 @@ sub downloadfile {
   #
   
 sub http_follow {
-  my ($dlinfo, $redirections) = @_;
-  my ($httpsock, $header);
+  my ($dlinit, $redirections) = @_;
+  my ($httpsock, $header, $code);
 
-  dbugout("HTTP_FOLLOW: getting ", urlstring($dlinfo)) if $debug;
+  dbugout("HTTP_FOLLOW: getting ", urlstring($dlinit)) if $debug;
 
-  set_http_proxy($dlinfo);
+  $dlinfo = set_http_proxy($dlinit);
   ($httpsock, $header) = http_start($dlinfo);
 
-  if ($header =~ /^http\/\S+\s+(\d+)\s+(.+?)\s*$/im) {
-    $httpcode = $1;
-    $httpresponse = $2;
-  } else {
-    close $httpsock;
-    choke("Malformed HTTP response from $dlinfo->{host}");
-  }
-
-  if ($httpcode eq '200') {
+  $code = $header->{status_code};
+  if ($code eq '200') {
     return ($httpsock, $header, $dlinfo);
-  } elsif ($httpcode =~ /^30[12357]$/) {
+  } elsif ($code =~ /^30[12357]$/) {
     $redirections ||= 0;
     if ($redirections > 9) {
       choke("server issued $redirections HTTP redirections; quitting");
-    } elsif ($header =~ /^location:\s+(\S+)/im) {
-      my $newurl = parse_url($1);
-      if ($$newurl{protocol} eq 'http') {
+    } elsif (exists $header->{location}) {
+      my $newurl = parse_url($header->{location});
+      if ($newurl->{protocol} eq 'http') {
         return http_follow($newurl, ++$redirections);
       } else {
         return (undef, $header, $newurl);
       }
     } else {
-      choke("$dlinfo->{host} replied \"$httpcode $httpresponse\" but no Location: can be found");
+      choke("$dlinfo->{host} replied \"$header->{status_line}\" but no Location: can be found");
     }
+  } elsif ($code eq '407') {
+    if (exists $dlinfo->{proxy_auth}) {
+      choke("Authentication to proxy at $dlinfo->{server}:$dlinfo->{port} failed.");
+    }
+    $dlinit->{proxy_auth} = get_proxy_auth($dlinfo, $header);
+    return http_follow($dlinit, $redirections);
   } else {
-    choke("$dlinfo->{host} replied \"$httpcode $httpresponse\"");
+    choke("$dlinfo->{host} replied \"$header->{status_line}\"");
   }
 }
 
@@ -462,8 +459,8 @@ sub http_follow {
 
 sub http_start {
   my $dlinfo = shift;
-
-  my ($iaddr, $paddr, $rqheader, $header, $line);
+  my $crlf = "\015\012";  # RFC 1945 section 2.2 ("end-of-line marker")
+  my ($iaddr, $paddr, $rqheader, $dbrqheader, $header, $line, $hparsed);
 
   dbugout("HTTP_START: getting ", urlstring($dlinfo)) if $debug;
 
@@ -482,27 +479,44 @@ sub http_start {
 
   select((select(HTTPSOCK), $| = 1)[0]);
 
-  $rqheader =
-    "GET $dlinfo->{location} HTTP/1.0\n" .
-    "User-Agent: wmtheme/$VERSION ($uname_srm)\n" .
-    "Host: $dlinfo->{host}\n" .
-    "Accept: */*\n\n";
+  $dbrqheader = $rqheader =
+    "GET $dlinfo->{location} HTTP/1.0$crlf" .
+    "User-Agent: wmtheme/$VERSION ($uname_srm)$crlf" .
+    "Host: $dlinfo->{host}$crlf";
+
+    # Obscure the authorization code.  Mostly useless, but if someone
+    # felt it should be kept a secret, they might be entering the password
+    # by hand each time.  Honor their desire in debug mode with XXXXXX.
+
+  if (exists $dlinfo->{proxy_auth}) {
+    $rqheader .= "${pa_str}$dlinfo->{proxy_auth}$crlf";
+    $dbrqheader .= "${pa_str}XXXXXX$crlf";
+  }
+  $rqheader .= "Accept: */*$crlf$crlf";
+  $dbrqheader .= "Accept: */*$crlf";
 
   dbugout(" ************  REQUEST  ************",
-    "$rqheader ==================================") if $debug;
+    "$dbrqheader ==================================") if $debug;
 
   print HTTPSOCK $rqheader;
 
+    # Sorry, won't be supporting HTTP 0.9 "Simple response"
   $header = '';
   while ($line = <HTTPSOCK>) {
-    $header .= $line;
     last if $line =~ /^[\r\s]*$/;
+    $header .= $line;
   }
 
   dbugout(" ++++++++++++  RESPONSE  ++++++++++++",
     "$header ----------------------------------") if $debug;
 
-  (\*HTTPSOCK, $header);
+  $hparsed = parse_header($header);
+  unless (exists $hparsed->{status_code}) {
+    close HTTPSOCK;
+    choke("Malformed HTTP response from $dlinfo->{host}");
+  }
+
+  (\*HTTPSOCK, $hparsed);
 }
 
 
@@ -516,7 +530,7 @@ sub parse_url {
   my %parsed;
 
   dbugout("PARSE_URL: Examining $url") if $debug;
-  if ($url =~ m|^(\w+)://($hostmatch)(?::(\d+))?(/.*)|) {
+  if ($url =~ m!^(\w+)://($hostmatch)(?::(\d+))?(/.*)!) {
     $parsed{protocol} = lc $1;
     $parsed{server}   = $2;
     $parsed{host}     = $2;
@@ -531,40 +545,147 @@ sub parse_url {
   \%parsed;
 }
 
-  # See http://www.w3.org/Daemon/User/Proxies/ProxyClients.html
+sub parse_header {
+  my $header = shift;
+  my ($line, $last);
+  my %result = ();
+
+    # RFC 1945 sections 3.1 & 6.1
+  if ($header =~ s/^HTTP\/\d+\.\d+\s+((\d\d\d)\s+(.+))//) {
+    @result{'status_line', 'status_code', 'status_reason'} = ($1, $2, $3);
+
+    while ($header =~ /^([ \t])?(.+?)\015?$/mg) {
+      $line = $2;
+      if (defined $1) {
+        $result{$last} .= $line if $last;
+  
+        # Section 4.2
+      } elsif ($line =~ /^([^:]+):\s*(.+)/) {
+        $last = lc $1;
+        $result{$last} = $2;
+      }
+    }
+  }
+
+  dbugout("PARSE_HEADER:",
+    map "\t$_ => $result{$_}", sort keys %result) if $debug;
+
+  \%result;
+}
+
+  ###########################################################################
+  #
+  #  SET_HTTP_PROXY
+  #
+  #  Examines environment variables to see if a proxy must be used.  If so,
+  #  modifies the provided $dlinfo to use the proxy.  Also sets its
+  #  authorization, if known.
+  #  But if there is no proxy, $dlinfo will be returned unmodified.
+  #
+  # See:
+  #  http://www.w3.org/Daemon/User/Proxies/ProxyClients.html
+  #  http://www.freesoft.org/CIE/RFC/2068/195.htm
+  #  http://www.freesoft.org/CIE/RFC/2068/114.htm
 
 sub set_http_proxy {
   my $dlinfo = shift;
-  my ($proxy, $np);
+  my ($proxy, $pport, $np);
+  my $dlout = {};
 
   if ($np = $ENV{no_proxy} || $ENV{NO_PROXY}) {
     foreach (split /[\s,]+/, $np) {
-      return if $dlinfo->{server} =~ /\b\Q$_\E$/;
+      return $dlinfo if $dlinfo->{server} =~ /\b\Q$_\E$/;
     }
   }
 
   $proxy = $ENV{http_proxy} || $ENV{HTTP_PROXY} || $ENV{all_proxy} ||
-           $ENV{ALL_PROXY} || return;
+           $ENV{ALL_PROXY} || return $dlinfo;
 
-  if ($proxy =~ m!^(?i:http://)?($hostmatch)(?::(\d+))?!) {
-    $dlinfo->{location} =
+  if ($proxy =~ m!^(?i:http://)?(?:([^:]+:[^@]+)@)?($hostmatch)(?::(\d+))?!) {
+    $dlout->{location} =
       "$dlinfo->{protocol}://$dlinfo->{server}:$dlinfo->{port}$dlinfo->{location}";
-    $dlinfo->{host}     = $dlinfo->{server};
-    $dlinfo->{protocol} = 'http';
-    $dlinfo->{server}   = $1;
-    $dlinfo->{port}     = $2 || 80;
-    dbugout("HTTP_PROXY:  host=$dlinfo->{server}  port=$dlinfo->{port}")
+    $dlout->{host}     = $dlinfo->{server};
+    $dlout->{protocol} = 'http';
+    $dlout->{server}   = $2;
+    $dlout->{port}     = $3 || 80;
+
+      # Set authorization if it's (1) cached in %proxy_auth or
+      # (2) avaliable in %ENV.  This is not necessary but it can save time
+      # by preventing a 407 response from the proxy.
+
+    $pport = "$2:$dlout->{port}";
+    if ($1) {
+      $proxy_auth{$pport} = encode64($1);
+    } elsif (exists $ENV{proxy_user} and exists $ENV{proxy_pass}) {
+      $proxy_auth{$pport} = encode64("$ENV{proxy_user}:$ENV{proxy_pass}");
+    }
+
+    $dlout->{proxy_auth} = $proxy_auth{$pport} if exists $proxy_auth{$pport};
+
+    dbugout("HTTP_PROXY:  host=$dlout->{server}  port=$dlout->{port}")
       if $debug;
+    return $dlout;
   } else {
     complain("HTTP_PROXY: can't parse proxy \"$proxy\"");
   }
+  $dlinfo;
 }
 
+  ###########################################################################
+  #
+  #  GET_PROXY_AUTH
+  #
+  #  This sub will be called when a request has been made to a proxy, but it
+  #  responded with 407.  Authorization _would_ have been included in the
+  #  initial request if it were known, so this sub is generally expected
+  #  prompt the user.
 
-  #########################################################################
-  #
-  #  URLSTRING
-  #
+sub get_proxy_auth {
+  my ($dlinfo, $header) = @_;
+  my $pinfo = "$dlinfo->{server}:$dlinfo->{port}";
+  my $user;
+
+  if (exists $header->{'proxy-authenticate'} and
+      $header->{'proxy-authenticate'} =~ /^(\w+)/) {
+    choke("Can't authenticate to proxy at $pinfo: scheme \"$1\" unsupported")
+      unless lc $1 eq 'basic';
+  } else {
+    choke("Can't authenticate to proxy at $pinfo: no challenge!");
+  }
+
+  if (exists $ENV{proxy_user}) {
+      # proxy_user and proxy_pass aren't both set or we would have used
+      # them during sub set_http_proxy
+    $user = $ENV{proxy_user};
+    print "\nEnter password for proxy at $pinfo: ";
+  } else {
+    print "\nEnter username for proxy at $pinfo: ";
+    chomp($user = <STDIN>);
+    choke("Cancelled.") if $user eq '';
+    print "Enter password: ";
+  }
+
+  $proxy_auth{$pinfo} = encode64($user . ':' . getpass());
+}
+
+sub getpass {
+  my $pass;
+
+  system('stty', '-echo');
+  chomp($pass = <STDIN>);
+  print "\n";
+  system('stty', 'echo');
+  $pass;
+}
+
+sub encode64 {
+  my ($data, $out, $pad) = (shift, '');
+
+  chomp($out .= substr(pack('u', $1), 1)) while $data =~ /(.{1,45})/gs;
+  $out =~ y/` -_/AA-Za-z0-9+\//;
+  $out =~ s/.{$pad}$/'=' x $pad/e if $pad = 3 - (length($data) % 3 || 3);
+  $out;
+}
 
 sub urlstring {
   my $url = shift;
@@ -645,5 +766,13 @@ sub makedlgauge {
   $gauge;
 }
 
+BEGIN {
+  read_userconfig();
+  foreach my $proxy (keys %{$config_wm{proxy_auth}}) {
+    dbugout("WM_DOWNLOAD: noting proxy authentication for $proxy") if $debug;
+    $proxy_auth{$proxy} = encode64($config_wm{proxy_auth}{$proxy});
+  }
+}
+
 1;
 
